@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@ import multiprocessing.pool
 import functools
 import urllib.request
 import urllib.error
+import socks
 
 import sabnzbd
 import sabnzbd.cfg
@@ -64,6 +65,13 @@ def addresslookup6(myhost):
     return socket.getaddrinfo(myhost, 80, socket.AF_INET6)
 
 
+def active_socks5_proxy():
+    """Return the active proxy"""
+    if socket.socket == socks.socksocket:
+        return "%s:%s" % socks.socksocket.default_proxy[1:3]
+    return None
+
+
 def localipv4():
     try:
         with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s_ipv4:
@@ -85,8 +93,9 @@ def publicipv4():
         ipv4_found = False
         # we only want IPv4 resolving, so socket.AF_INET:
         result = addresslookup4(sabnzbd.cfg.selftest_host())
-    except (socket.error, multiprocessing.context.TimeoutError):
+    except (ValueError, socket.error, multiprocessing.context.TimeoutError):
         # something very bad: no urllib2, no resolving of selftest_host, no network at all
+        # Or strange DSM problem: https://github.com/sabnzbd/sabnzbd/issues/2008
         return public_ipv4
 
     # we got one or more IPv4 address(es), so let's connect to them
@@ -124,6 +133,6 @@ def ipv6():
             # IPv6 prefix for documentation purpose
             s_ipv6.connect(("2001:db8::8080", 80))
             ipv6_address = s_ipv6.getsockname()[0]
-    except socket.error:
+    except:
         ipv6_address = None
     return ipv6_address
