@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -48,7 +48,7 @@ from sabnzbd.constants import (
     STOP_PRIORITY,
     VERIFIED_FILE,
     Status,
-    IGNORED_FOLDERS,
+    IGNORED_FILES_AND_FOLDERS,
     QNFO,
     DIRECT_WRITE_TRIGGER,
 )
@@ -76,7 +76,7 @@ class NzbQueue:
         if repair < 2:
             # Try to process the queue file
             try:
-                data = sabnzbd.load_admin(QUEUE_FILE_NAME)
+                data = sabnzbd.filesystem.load_admin(QUEUE_FILE_NAME)
                 if data:
                     queue_vers, nzo_ids, _ = data
                     if not queue_vers == QUEUE_VERSION:
@@ -99,11 +99,11 @@ class NzbQueue:
             path = get_admin_path(folder, future=False)
 
             # Try as normal job
-            nzo = sabnzbd.load_data(_id, path, remove=False)
+            nzo = sabnzbd.filesystem.load_data(_id, path, remove=False)
             if not nzo:
                 # Try as future job
                 path = get_admin_path(folder, future=True)
-                nzo = sabnzbd.load_data(_id, path)
+                nzo = sabnzbd.filesystem.load_data(_id, path)
             if nzo:
                 self.add(nzo, save=False, quiet=True)
                 folders.append(folder)
@@ -117,7 +117,7 @@ class NzbQueue:
                 path, nzo_id = os.path.split(item)
                 if nzo_id not in self.__nzo_table:
                     if nzo_id.startswith("SABnzbd_nzo"):
-                        nzo = sabnzbd.load_data(nzo_id, path, remove=True)
+                        nzo = sabnzbd.filesystem.load_data(nzo_id, path, remove=True)
                         if nzo:
                             self.add(nzo, save=True)
                     else:
@@ -154,7 +154,7 @@ class NzbQueue:
         # Repair unregistered folders
         for folder in globber_full(cfg.download_dir.get_path()):
             name = os.path.basename(folder)
-            if os.path.isdir(folder) and name not in registered and name not in IGNORED_FOLDERS:
+            if os.path.isdir(folder) and name not in registered and name not in IGNORED_FILES_AND_FOLDERS:
                 if action:
                     logging.info("Repairing job %s", folder)
                     self.repair_job(folder)
@@ -179,17 +179,19 @@ class NzbQueue:
         if getattr(new_nzb, "filename", None):
             remove_all(admin_path, "*.gz", keep_folder=True)
             logging.debug("Repair job %s with new NZB (%s)", name, new_nzb.filename)
-            _, nzo_ids = sabnzbd.add_nzbfile(new_nzb, nzbname=name, reuse=repair_folder, password=password)
+            _, nzo_ids = sabnzbd.nzbparser.add_nzbfile(new_nzb, nzbname=name, reuse=repair_folder, password=password)
         else:
             # Was this file already post-processed?
-            verified = sabnzbd.load_data(VERIFIED_FILE, admin_path, remove=False)
+            verified = sabnzbd.filesystem.load_data(VERIFIED_FILE, admin_path, remove=False)
             filenames = []
             if not verified or not all(verified[x] for x in verified):
                 filenames = globber_full(admin_path, "*.gz")
 
             if filenames:
                 logging.debug("Repair job %s by re-parsing stored NZB", name)
-                _, nzo_ids = sabnzbd.add_nzbfile(filenames[0], nzbname=name, reuse=repair_folder, password=password)
+                _, nzo_ids = sabnzbd.nzbparser.add_nzbfile(
+                    filenames[0], nzbname=name, reuse=repair_folder, password=password
+                )
             else:
                 try:
                     logging.debug("Repair job %s without stored NZB", name)
@@ -237,16 +239,16 @@ class NzbQueue:
         nzo_ids = []
         # Aggregate nzo_ids and save each nzo
         for nzo in self.__nzo_list[:]:
-            if not nzo.is_gone():
+            if not nzo.removed_from_queue:
                 nzo_ids.append(os.path.join(nzo.work_name, nzo.nzo_id))
                 if save_nzo is None or nzo is save_nzo:
                     if not nzo.futuretype:
                         # Also includes save_data for NZO
                         nzo.save_to_disk()
                     else:
-                        sabnzbd.save_data(nzo, nzo.nzo_id, nzo.admin_path)
+                        sabnzbd.filesystem.save_data(nzo, nzo.nzo_id, nzo.admin_path)
 
-        sabnzbd.save_admin((QUEUE_VERSION, nzo_ids, []), QUEUE_FILE_NAME)
+        sabnzbd.filesystem.save_admin((QUEUE_VERSION, nzo_ids, []), QUEUE_FILE_NAME)
 
     def set_top_only(self, value):
         self.__top_only = value
@@ -327,7 +329,7 @@ class NzbQueue:
     @NzbQueueLocker
     def add(self, nzo: NzbObject, save: bool = True, quiet: bool = False) -> str:
         if not nzo.nzo_id:
-            nzo.nzo_id = sabnzbd.get_new_id("nzo", nzo.admin_path, self.__nzo_table)
+            nzo.nzo_id = sabnzbd.filesystem.get_new_id("nzo", nzo.admin_path, self.__nzo_table)
 
         # If no files are to be downloaded anymore, send to postproc
         if not nzo.files and not nzo.futuretype:
@@ -336,7 +338,7 @@ class NzbQueue:
 
         # Reset try_lists, markers and evaluate the scheduling settings
         nzo.reset_try_list()
-        nzo.deleted = False
+        nzo.removed_from_queue = False
         priority = nzo.priority
         if sabnzbd.Scheduler.analyse(False, priority):
             nzo.status = Status.PAUSED
@@ -393,12 +395,10 @@ class NzbQueue:
             logging.info("[%s] Removing job %s", caller_name(), nzo.final_name)
 
             # Set statuses
-            nzo.deleted = True
-            if cleanup and not nzo.is_gone():
-                nzo.status = Status.DELETED
+            nzo.removed_from_queue = True
             self.__nzo_list.remove(nzo)
-
             if cleanup:
+                nzo.status = Status.DELETED
                 nzo.purge_data(delete_all_data=delete_all_data)
             self.save(False)
             return nzo_id
@@ -735,47 +735,49 @@ class NzbQueue:
         nzf = article.nzf
         nzo = nzf.nzo
 
-        if nzf.deleted:
-            logging.debug("Discarding article %s, no longer in queue", article.article)
+        if nzo.pp_or_finished or nzf.deleted:
+            logging.debug("Discarding article for file %s: deleted or already post-processing", nzf.filename)
+            # If this file is needed later (par2 file added back to queue), it would be damaged because
+            # we discard this article. So we reset it to be picked up again if needed.
+            # But not reset all articles, as it could cause problems for articles still attached to a server.
+            article.reset_try_list()
+            nzf.reset_try_list()
             return
 
         articles_left, file_done, post_done = nzo.remove_article(article, success)
 
-        if nzo.is_gone():
-            logging.debug("Discarding article for file %s, no longer in queue", nzf.filename)
-        else:
-            # Write data if file is done or at trigger time
-            if file_done or (articles_left and (articles_left % DIRECT_WRITE_TRIGGER) == 0):
-                if not nzo.precheck:
-                    # Only start decoding if we have a filename and type
-                    # The type is only set if sabyenc could decode the article
-                    if nzf.filename and nzf.type:
-                        sabnzbd.Assembler.process(nzo, nzf, file_done)
-                    elif nzf.filename.lower().endswith(".par2"):
-                        # Broken par2 file, try to get another one
-                        nzo.promote_par2(nzf)
+        # Write data if file is done or at trigger time
+        if file_done or (articles_left and (articles_left % DIRECT_WRITE_TRIGGER) == 0):
+            if not nzo.precheck:
+                # Only start decoding if we have a filename and type
+                # The type is only set if sabyenc could decode the article
+                if nzf.filename and nzf.type:
+                    sabnzbd.Assembler.process(nzo, nzf, file_done)
+                elif nzf.filename.lower().endswith(".par2"):
+                    # Broken par2 file, try to get another one
+                    nzo.promote_par2(nzf)
 
-            # Save bookkeeping in case of crash
-            if file_done and (nzo.next_save is None or time.time() > nzo.next_save):
-                nzo.save_to_disk()
-                sabnzbd.BPSMeter.save()
-                if nzo.save_timeout is None:
-                    nzo.next_save = None
-                else:
-                    nzo.next_save = time.time() + nzo.save_timeout
+        # Save bookkeeping in case of crash
+        if file_done and (nzo.next_save is None or time.time() > nzo.next_save):
+            nzo.save_to_disk()
+            sabnzbd.BPSMeter.save()
+            if nzo.save_timeout is None:
+                nzo.next_save = None
+            else:
+                nzo.next_save = time.time() + nzo.save_timeout
 
-            # Remove post from Queue
-            if post_done:
-                nzo.set_download_report()
-                self.end_job(nzo)
+        # Remove post from Queue
+        if post_done:
+            nzo.set_download_report()
+            self.end_job(nzo)
 
     @NzbQueueLocker
     def end_job(self, nzo: NzbObject):
         """Send NZO to the post-processing queue"""
         # Notify assembler to call postprocessor
-        if not nzo.deleted:
+        if not nzo.removed_from_queue:
             logging.info("[%s] Ending job %s", caller_name(), nzo.final_name)
-            nzo.deleted = True
+            nzo.removed_from_queue = True
             if nzo.precheck:
                 nzo.save_to_disk()
                 # Check result
