@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,16 +25,15 @@ import time
 import logging
 import queue
 import urllib.request
-import urllib.error
 import urllib.parse
 from http.client import IncompleteRead, HTTPResponse
 from mailbox import Message
 from threading import Thread
 import base64
-from typing import Tuple, Optional
+from typing import Tuple, Optional, Union
 
 import sabnzbd
-from sabnzbd.constants import DEF_TIMEOUT, FUTURE_Q_FOLDER, VALID_NZB_FILES, Status, VALID_ARCHIVES
+from sabnzbd.constants import DEF_TIMEOUT, FUTURE_Q_FOLDER, VALID_NZB_FILES, Status, VALID_ARCHIVES, DEFAULT_PRIORITY
 import sabnzbd.misc as misc
 import sabnzbd.filesystem
 import sabnzbd.cfg as cfg
@@ -64,8 +63,6 @@ class URLGrabber(Thread):
     def __init__(self):
         super().__init__()
         self.queue: queue.Queue[Tuple[Optional[str], Optional[NzbObject]]] = queue.Queue()
-        for url_nzo_tup in sabnzbd.NzbQueue.get_urls():
-            self.queue.put(url_nzo_tup)
         self.shutdown = False
 
     def add(self, url: str, future_nzo: NzbObject, when: Optional[int] = None):
@@ -88,7 +85,11 @@ class URLGrabber(Thread):
         self.queue.put((None, None))
 
     def run(self):
-        self.shutdown = False
+        # Read all URL's to grab from the queue
+        for url_nzo_tup in sabnzbd.NzbQueue.get_urls():
+            self.queue.put(url_nzo_tup)
+
+        # Start fetching
         while not self.shutdown:
             # Set NzbObject object to None so reference from this thread
             # does not keep the object alive in the future (see #1628)
@@ -117,7 +118,7 @@ class URLGrabber(Thread):
                 if future_nzo:
                     # If nzo entry deleted, give up
                     try:
-                        deleted = future_nzo.deleted
+                        deleted = future_nzo.removed_from_queue
                     except AttributeError:
                         deleted = True
                     if deleted:
@@ -250,7 +251,9 @@ class URLGrabber(Thread):
 
                 # If no filename, make one
                 if not filename:
-                    filename = sabnzbd.get_new_id("url", os.path.join(cfg.admin_dir.get_path(), FUTURE_Q_FOLDER))
+                    filename = sabnzbd.filesystem.get_new_id(
+                        "url", os.path.join(cfg.admin_dir.get_path(), FUTURE_Q_FOLDER)
+                    )
 
                 # Write data to temp file
                 path = os.path.join(cfg.admin_dir.get_path(), FUTURE_Q_FOLDER, filename)
@@ -259,7 +262,7 @@ class URLGrabber(Thread):
 
                 # Check if nzb file
                 if sabnzbd.filesystem.get_ext(filename) in VALID_ARCHIVES + VALID_NZB_FILES:
-                    res, _ = sabnzbd.add_nzbfile(
+                    res, _ = sabnzbd.nzbparser.add_nzbfile(
                         path,
                         pp=pp,
                         script=script,
@@ -390,3 +393,40 @@ def filename_from_content_disposition(content_disposition: str) -> Optional[str]
         filename = os.path.basename(filename).lstrip(".").strip()
         if filename:
             return filename
+
+
+def add_url(
+    url: str,
+    pp: Optional[Union[int, str]] = None,
+    script: Optional[str] = None,
+    cat: Optional[str] = None,
+    priority: Optional[Union[int, str]] = DEFAULT_PRIORITY,
+    nzbname: Optional[str] = None,
+    password: Optional[str] = None,
+):
+    """Add NZB based on a URL, attributes optional"""
+    if not url.lower().startswith("http"):
+        return
+    if not pp or pp == "-1":
+        pp = None
+    if script and script.lower() == "default":
+        script = None
+    if cat and cat.lower() == "default":
+        cat = None
+    logging.info("Fetching %s", url)
+
+    # Add feed name if it came from RSS
+    msg = T("Trying to fetch NZB from %s") % url
+    if nzbname:
+        msg = "%s - %s" % (nzbname, msg)
+
+    # Generate the placeholder
+    future_nzo = sabnzbd.NzbQueue.generate_future(msg, pp, script, cat, url=url, priority=priority, nzbname=nzbname)
+
+    # Set password
+    if not future_nzo.password:
+        future_nzo.password = password
+
+    # Get it!
+    sabnzbd.URLGrabber.add(url, future_nzo)
+    return future_nzo.nzo_id
